;;;;; ezxdisp.scm

(module ezxdisp
        
        (ezx-init
         ezx-quit

         ezx-redraw
         ezx-resize
         ezx-wipe
         ezx-wipe-layer
         ezx-select-layer
         ezx-set-background
         ezx-window-name
         ezx-isclosed
         ezx-sensebutton
         ezx-pushbutton
         
         ezx-point-2d
         ezx-line-2d
         ezx-lines-2d-lolevel
         ezx-poly-2d-lolevel
         ezx-str-2d
         ezx-rect-2d
         ezx-fillrect-2d
         ezx-circle-2d
         ezx-fillcircle-2d
         ezx-arc-2d
         ezx-fillarc-2d
         
         ezx-c3d-to-2d
         ezx-line-3d
         ezx-set-light-3d
         ezx-set-view-3d
         ezx-str-3d
         ezx-poly-3d-lolevel
         ezx-circle-3d

         make-ezx-color
         ezx-next-event
         ezx-lines-2d
         ezx-poly-2d
         ezx-poly-3d

         ezx:BUTTON-PRESS
         ezx:BUTTON-RELEASE
         ezx:KEY-PRESS
         ezx:KEY-RELEASE
         ezx:MOTION-NOTIFY
         ezx:CLOSE
         )

	(import scheme (chicken base)
		(except (chicken foreign) foreign-declare)
                srfi-4 bind)

(define-record ezx ptr)

(bind-type ezxp (c-pointer "ezx_t") ezx-ptr make-ezx)
(bind-type ezxcp f64vector)
(bind-options export-constants: #t)
(bind-rename/pattern "_" "-")
(bind-rename/pattern "EZX-" "ezx:")

#>
#include "ezxdisp.h"

static int
get_next_event(ezx_t *e, int *x, int *y, unsigned int *state, unsigned int *kb)
{
  ezx_event_t ev;

  ezx_next_event(e, &ev);

  switch(ev.type) {
  case EZX_BUTTON_PRESS:
  case EZX_BUTTON_RELEASE:
    *kb = ev.button.b;
    *x = ev.button.x;
    *y = ev.button.y;
    *state = ev.button.state;
    break;

  case EZX_KEY_PRESS:
  case EZX_KEY_RELEASE:
    *kb = ev.key.k & 0xffff;
    *x = ev.key.x;
    *y = ev.key.y;
    *state = ev.key.state;
    break;

  case EZX_MOTION_NOTIFY:
    *x = ev.motion.x;
    *y = ev.motion.y;
    *state = ev.motion.state;
    break;
  }

  return ev.type;
}
<#


(bind #<<EOF

enum event_type {
    EZX_BUTTON_PRESS,
    EZX_BUTTON_RELEASE,
    EZX_KEY_PRESS,
    EZX_KEY_RELEASE,
    EZX_MOTION_NOTIFY,
    EZX_CLOSE
  } ;

ezxp ezx_init(int size_x, int size_y, char *window_name);
void   ezx_quit(ezxp e);
void   ezx_redraw(ezxp e);
void   ezx_resize(ezxp e, int size_x, int size_y);
void   ezx_wipe(ezxp e);
void   ezx_wipe_layer(ezxp e, int lay);
void   ezx_select_layer(ezxp e, int lay);
void   ezx_set_background(ezxp e, double * col);
void   ezx_window_name(ezxp e, char *window_name);
int    ezx_isclosed(ezxp e);
int    ezx_sensebutton(ezxp e, ___out int *x, ___out int *y);
int    ezx_pushbutton(ezxp e, ___out int *x, ___out int *y);

// for 2D graphics
void   ezx_point_2d(ezxp e, int x, int y, double * col);
void   ezx_line_2d(ezxp e, int x0, int y0, int x1, int y1, double * col, int width);
void   ezx_lines_2d_lolevel(ezxp e, int *points, int npoints, double * col, int width);
void   ezx_poly_2d_lolevel(ezxp e,int *points, int npoints, double * col);
void   ezx_str_2d(ezxp e, double x0, double y0, char *str, double * col);
void   ezx_rect_2d(ezxp e, int x0, int y0, int x1, int y1, double * col, int width);
void   ezx_fillrect_2d(ezxp e, double x0, double y0, double x1, double y1, double * col);
void   ezx_circle_2d(ezxp e, int x, int y, int r, double * col, int width);
void   ezx_fillcircle_2d(ezxp e, int x0, int y0, int r, double * col);
void   ezx_arc_2d(ezxp e, int x, int y, int w, int h, double angle1, double angle2, double * col, int width);
void   ezx_fillarc_2d(ezxp e, int x, int y, int w, int h, double angle1, double angle2, double * col);

// for 3D graphics
void   ezx_c3d_to_2d(ezxp e, double sx, double sy, double sz, ___out double *dx, ___out double *dy);
void   ezx_line_3d(ezxp e, double x0, double y0, double z0, double x1, double y1, double z1, double * col, int width);
void   ezx_set_light_3d(ezxp e, double ex, double ey, double ez);
void   ezx_set_view_3d(ezxp e, double ex, double ey, double ez, double vx, double vy, double vz, double m);
void   ezx_str_3d(ezxp e, double x0, double y0, double z0, char *str, double * col);
void   ezx_poly_3d_lolevel(ezxp e, double *points, double hx, double hy, double hz, int npoints, double * col);
void   ezx_circle_3d(ezxp e, double x0, double y0, double z0, double r, double * col);

int get_next_event(ezxp e, ___out int *x, ___out int *y, ___out unsigned int *state, ___out unsigned int *kb);

EOF
)

(define (make-ezx-color r g b)
  (f64vector r g b) )

(define ezx-next-event get-next-event)

(define (ezx-lines-2d ezx points color width)
  (ezx-lines-2d-lolevel ezx points (s32vector-length points) color width))
  
(define (ezx-poly-2d ezx points color)
  (ezx-poly-2d-lolevel ezx points (s32vector-length points) color))
  
(define (ezx-poly-3d ezx points hx hy hz color)
  (ezx-poly-3d-lolevel ezx points hx hy hz (s32vector-length points) color))
  

)
